import mcmd.config.config as config
from mcmd import io
from mcmd.client.molgenis_client import login, ResourceType, post, ensure_resource_exists
from mcmd.io import highlight


# =========
# Arguments
# =========


def arguments(subparsers):
    p_enable = subparsers.add_parser('enable',
                                     help='Enable resources/functionality',
                                     description="Run 'mcmd enable rls -h' to view the help for those sub-commands")
    p_enable_subparsers = p_enable.add_subparsers(dest="type")

    p_enable_rls = p_enable_subparsers.add_parser('rls',
                                                  help='Enables row level security on an entity type')
    p_enable_rls.set_defaults(func=enable_rls,
                              write_to_history=True)
    p_enable_rls.add_argument('entity',
                              type=str,
                              help="The entity type to secure")


# =======
# Methods
# =======

@login
def enable_rls(args):
    io.start('Enabling row level security on entity type %s' % highlight(args.entity))

    ensure_resource_exists(args.entity, ResourceType.ENTITY_TYPE)
    post(config.api('rls'), data={'id': args.entity,
                                  'rlsEnabled': True})



"""
Provides access to the configuration.
"""

import operator
from functools import reduce
from pathlib import Path
from urllib.parse import urljoin

from mcmd.utils import McmdError

_config = None


def set_config(config):
    global _config
    if _config:
        raise ValueError('config already set')

    _config = config


def get(*args):
    prop = _config
    for at in list(args):
        prop = prop[at]
    return prop


def username():
    return _get_selected_host_auth()['username']


def password():
    return _get_selected_host_auth()['password']


def git_paths():
    root = _config['git']['root']
    if root is None or len(root) == 0:
        return []
    else:
        root_path = Path(root)
        paths = _config['git']['paths']
        return [root_path.joinpath(path) for path in paths]


def api(endpoint):
    """Returns the combination of the host's url and the API endpoint."""
    url = _config['host']['selected']
    return urljoin(url, get('api', endpoint))


def has_option(*args):
    try:
        reduce(operator.getitem, list(args), _config)
        return True
    except KeyError:
        return False


def _get_selected_host_auth():
    selected = _config['host']['selected']
    hosts = _config['host']['auth']
    for host in hosts:
        if host['url'] == selected:
            return host

    raise McmdError("The selected host doesn't exist.")
